/*
 * CommandQueue.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import  java.util.*;

/**
 * A class encapsulating a queue of Commands to be processed by the Robot.
 * Commands put into the queue are returned in a first-in-first-out order.
 * Basically, this is a hack using a Hashtable and two keys: an input key 
 * used to put Commands into the queue, and an output key to retrieve the
 * Commands out of the queue.  If the output key is greater than or equal to
 * the input key, the queue is deemed empty.
 *
 * @author Brendon Wilson
 * @version 1.0 alpha
 *
 * @see SimulationEngine
 * @see Command
 */
public class CommandQueue
{
    /** A Hashtable containing the Commands for the Simulation. */
    private Hashtable queue;

    /** The key used to put Commands into the Hashtable. */
    private int in;

    /** The key used to remove Commands from the Hashtable. */
    private int out;


    /**
	 * Creates and initializes a new queue.
	 */
    public CommandQueue()
    {
        queue = new Hashtable();
        in = 1;
        out = 1;
    }

    /**
     * Adds a Command into the FIFO queue.
     *
     * @param c a Command to be put into the queue.
     */
    public void putCommand(Command c)
    {
        queue.put(new Integer(in), c);
        in++;
    }

    /**
     * Retrieves the next Command from the FIFO queue.
     *
     * @return the next Command in the queue.
     */
    public Command getCommand()
    {
        Command c = (Command)queue.remove(new Integer(out));
        out++;

        return c;
    }

    /**
     * Returns true if the CommandQueue object is empty.
     *
     * @return indicates if the CommandQueue is empty.
     */
    public boolean isEmpty()
    {
        if (out >= in)
        {
            return true;
        }
        else
        {
            return false;
        }
    }
}