/*
 * DefinePositionCommand.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/**
 * A class that encapsulates a command to define a name for a particular 
 * position of the robot. This class is a concrete implementation of the 
 * Command class, and is a part of REMOTE's implementation of the Command 
 * Pattern, as demonstrated in Gamma et al (page 236).
 *
 * @author Brendon Wilson
 * @version 1.0 alpha
 * @see SimulationEngine
 * @see CommandQueue
 */
public class DefinePositionCommand extends Command
{
    /**
     * The name for the position.
     */
    private String name;

    
    /**
     * Creates a new DefinePositionCommand which will associate a name with a
     * particular position of the Robot at run-time.
     *
     * @param lineNumber the source line for this command.
     * @param name the name to associate with the Robot's position.
     */
    public DefinePositionCommand(int lineNumber, String name)
    {
        super(lineNumber);

        // Save the name.
        this.name = name;
    }

    /**
     * Instructs the Robot to associate the robot current's position with 
     * the given name.
     *
     * @exception thrown if an error occurs attempting to execute the command.
     */
    public void Execute() throws Exception
    {
        // Get the Robot instance.
        Robot theRobot = Robot.getInstance();

        // Get the table instance.
        PositionTable thePositionTable = PositionTable.getInstance();

        // Retrieve the joint variables from the Robot.
        JointVariables joints = (JointVariables) theRobot.getJointVariables().clone();

        // Save the joint variables in the position table.
        thePositionTable.putNamedPosition(name, joints);
    }
}