/*
 * IndexPair.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/**
 * A class to store a Joint index and a Degree index, which is used elsewhere
 * to associate a name with specific Joint and Degree objects in the Robot's
 * hierarchy;  using a Hashtable, the name is used as a key and the IndexPair
 * as the value, allowing recall of a specific Degree by name.
 *
 * @author Brendon Wilson
 * @version 1.0 alpha
 * 
 * @see Robot
 * @see Joint
 * @see Degree
 */
public class IndexPair implements Cloneable
{
    /** The index the Joint containing a specific Degree. */
    public int joint;

    /** The index of a specific Degree. */
    public int degree;


    /**
     * Creates a new IndexPair, given and index for the joint and the degree.
     *
     * @param joint the Joint index.
     * @param degree the Degree index.
     */
    public IndexPair(int joint, int degree)
    {
        this.joint = joint;
        this.degree = degree;
    }

    /**
     * Clones this IndexPair object.
     *
     * @return the Object copy of this object.
     */
    public Object clone()
    {
        return (Object) new IndexPair(joint, degree);
    }
}