/*
 * ModelParser.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import  javax.media.j3d.*;

/**
 * An abstract class to encapsulate the functionality required to properly
 * parse a model file representing a 3D physical object. Perhaps should
 * migrate this to com.sun.java.j3d.loaders.* package classes.
 *
 * @author Brendon Wilson
 * @version 1.0 alpha
 */
public abstract class ModelParser
{
    /**
     * Private constructor. Use <code>loadParser</code> to load a specific 
     * ModelParser instance instead of instantiating a ModelParser directly.
	 */
    protected ModelParser()
    {
        super();
    }

    /**
     * Parses the given filename to create the BranchGroup object.
     *
     * @param filename the name of the file to parse for 3D model information.
     * @exception ModelParserException indicates an error parsing Reader.
     */
    public abstract void read(String filename) throws ModelParserException;

    /**
     * Returns a 3D representation created from the Reader stream.
     *
     * @return the 3D representation object created from the file.
     * @exception ModelParserException indicates the representation is empty.
     */
    public abstract Group getModel() throws ModelParserException;

    /**
     * Loads the ModelParser class specified by the given String.
     *
     * @param s the name of the ModelParser class to be loaded.
     * @return an instance of the requested ModelParser class.
     * @exception ModelParserException indicates an error loading the parser.
     */
    public static final ModelParser loadParser(String s)
        throws ModelParserException
    {
        Class aClass;
        ModelParser aModelParser;

        try
        {
            aClass = Class.forName(s);
            aModelParser = (ModelParser) aClass.newInstance();

            return aModelParser;
        }
        catch (Exception e)
        {
			System.out.println("Ahh shit!");
            throw new ModelParserException(e.getMessage());
        }
    }
}