/*
 * MouseRotate.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.behaviors.mouse.MouseBehavior;

/**
 * A fix of the Sun utility MouseRotate behavior, in order to use a set of
 * behaviors that are similar to Trispectives.  The only difference between
 * this class and the original Sun version, is that the behavior no longer
 * uses the meta or alt buttons.
 * 
 * @author Brendon Wilson
 * @version alpha
 */
public class MouseRotate extends MouseBehavior
{
    private double x_angle;
    private double y_angle;
    private double x_factor;
    private double y_factor;


    /**
     * Creates a rotate behavior given the transform group.
     * @param transformGroup The transformGroup to operate on.
     */
    public MouseRotate(TransformGroup transformGroup)
    {
        super(transformGroup);
    }

    /**
     * Creates a default mouse rotate behavior.
     */
    public MouseRotate()
    {
        super(0);
    }

    /**
     * Creates a rotate behavior.
     * Note that this behavior still needs a transform
     * group to work on (use setTransformGroup(tg)) and
     * the transform group must add this behavior.
     * @param flags interesting flags (wakeup conditions).
     */
    public MouseRotate(int flags)
    {
        super(flags);
    }

    public void initialize()
    {
        super.initialize();
        x_angle = 0;
        y_angle = 0;
        x_factor = .03;
        y_factor = .03;

        if ((flags & INVERT_INPUT) == INVERT_INPUT)
        {
            invert = true;
            x_factor *= -1;
            y_factor *= -1;
        }
    }

    public void processStimulus(Enumeration criteria)
    {
        WakeupCriterion wakeup;
        AWTEvent[] event;
        int id;
        int dx, dy;

        while (criteria.hasMoreElements())
        {
            wakeup = (WakeupCriterion) criteria.nextElement();

            if (wakeup instanceof WakeupOnAWTEvent)
            {
                event = ((WakeupOnAWTEvent)wakeup).getAWTEvent();

                for (int i = 0; i < event.length; i++)
                {
                    processMouseEvent((MouseEvent)event[i]);

                    if (buttonPress)
                    {
                        id = event[i].getID();

                        if ((id == MouseEvent.MOUSE_DRAGGED) 
                            && !((MouseEvent)event[i]).isMetaDown() 
                            && !((MouseEvent)event[i]).isAltDown())
                        {
                            x = ((MouseEvent) event[i]).getX();
                            y = ((MouseEvent) event[i]).getY();
                            dx = x - x_last;
                            dy = y - y_last;

                            if (!reset)
                            {
                                x_angle = dy*y_factor;
                                y_angle = dx*x_factor;

                                transformX.rotX(x_angle);
                                transformY.rotY(y_angle);

                                transformGroup.getTransform(currXform);
                                Matrix4d mat =  new Matrix4d();
                                
                                // Remember old matrix
                                currXform.get(mat);
                                
                                // Translate to origin
                                currXform.setTranslation(new Vector3d(0.0, 0.0, 0.0));

                                if (invert)
                                {
                                    currXform.mul(currXform, transformX);
                                    currXform.mul(currXform, transformY);
                                }
                                else
                                {
                                    currXform.mul(transformX, currXform);
                                    currXform.mul(transformY, currXform);
                                }

                                // Set old translation back
                                Vector3d translation =  
                                    new Vector3d(mat.m03, mat.m13, mat.m23);
                                currXform.setTranslation(translation);
                                
                                // Update xform
                                transformGroup.setTransform(currXform);
                            }
                            else
                            {
                                reset = false;
                            }

                            x_last = x;
                            y_last = y;
                        }
                        else if (id == MouseEvent.MOUSE_PRESSED)
                        {
                            x_last = ((MouseEvent) event[i]).getX();
                            y_last = ((MouseEvent) event[i]).getY();
                        }
                    }
                }
            }
        }

        if (buttonPress || ((flags & MANUAL_WAKEUP) == 0)) wakeupOn(mouseCriterion);
    }
}