/*
 * MouseZoom.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.behaviors.mouse.MouseBehavior;

/**
 * MouseZoom is a Java3D behavior object that lets users control the 
 * Z axis translation of an object via a mouse drag motion with the second
 * mouse button. See MouseRotate for similar usage info. This is a fix of 
 * the Sun utility MouseRotate behavior, in order to use a set of mouse
 * behaviors that are similar to Trispectives.  The only difference between
 * this class and the original Sun version, is that the behavior no longer
 * uses the meta or alt buttons.
 * 
 * @author Brendon Wilson
 * @version alpha
 */
public class MouseZoom extends MouseBehavior
{
    private double z_factor;
    private double zoom_factor;
    private Vector3d translation;


    /**
     * Creates a zoom behavior given the transform group.
     * @param transformGroup The transformGroup to operate on.
     */
    public MouseZoom(TransformGroup transformGroup)
    {
        super(transformGroup);
        translation = new Vector3d();
    }

    /**
     * Creates a default mouse zoom behavior.
     */
    public MouseZoom()
    {
        super(0);
        translation = new Vector3d();
    }

    /**
     * Creates a zoom behavior.
     * Note that this behavior still needs a transform
     * group to work on (use setTransformGroup(tg)) and
     * the transform group must add this behavior.
     * @param flags
     */
    public MouseZoom(int flags)
    {
        super(flags);
        translation = new Vector3d();
    }

    public void initialize()
    {
        super.initialize();
        z_factor = .04;
        zoom_factor = .02;

        if ((flags & INVERT_INPUT) == INVERT_INPUT)
        {
            z_factor *= -1;
            zoom_factor *= -1;
            invert = true;
        }
    }

    public void processStimulus(Enumeration criteria)
    {
        WakeupCriterion wakeup;
        AWTEvent[] event;
        int id;
        int dx, dy;

        while (criteria.hasMoreElements())
        {
            wakeup = (WakeupCriterion) criteria.nextElement();

            if (wakeup instanceof WakeupOnAWTEvent)
            {
                event = ((WakeupOnAWTEvent) wakeup).getAWTEvent();

                for (int i =  0; i < event.length; i++)
                {
                    processMouseEvent((MouseEvent) event[i]);

                    if (buttonPress)
                    {
                        id =  event[i].getID();

                        if ((id == MouseEvent.MOUSE_DRAGGED) 
                            && !((MouseEvent) event[i]).isAltDown() 
                            && !((MouseEvent) event[i]).isMetaDown())
                        {
                            x = ((MouseEvent) event[i]).getX();
                            y = ((MouseEvent) event[i]).getY();
                            dx = x - x_last;
                            dy = y - y_last;

                            if (!reset)
                            {
                                transformGroup.getTransform(currXform);
                                translation.z =  dy*z_factor;
                                transformX.set(translation);

                                if (invert)
                                {
                                    currXform.mul(currXform, transformX);
                                }
                                else
                                {
                                    currXform.mul(transformX, currXform);
                                }

                                transformGroup.setTransform(currXform);
                            }
                            else
                            {
                                reset = false;
                            }

                            x_last = x;
                            y_last = y;
                        }
                        else if (id == MouseEvent.MOUSE_PRESSED)
                        {
                            x_last = ((MouseEvent) event[i]).getX();
                            y_last = ((MouseEvent) event[i]).getY();
                        }
                    }
                }
            }
        }

        if (buttonPress || ((flags & MANUAL_WAKEUP) == 0)) wakeupOn(mouseCriterion);
    }
}