/*
 * WaitCommand.java
 *
 * Copyright (c) 1996-1998 Brendon Wilson. All Rights Reserved.
 *
 * Brendon Wilson grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and binary 
 * code form, provided that i) this copyright notice and license appear on all
 * copies of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Brendon Wilson.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. BRENDON WILSON SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL BRENDON
 * WILSON BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF BRENDON WILSON HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/**
 * A class that encapsulates a command to pause the simulation. This class is a
 * concrete implementation of the Command class, and is a part of REMOTE's 
 * implementation of the Command Pattern, as demonstrated in Gamma et al (page 
 * 236). 
 *
 * @author Brendon Wilson
 * @version 1.0 alpha
 * @see SimulationEngine
 * @see CommandQueue
 */
public class WaitCommand extends Command
{
    /**
     * The amount of time to pause the simulation, in milliseconds.
     */
    private long waitTime;


    /**
     * Create a new WaitCommand, which will pause the simulation for the given
     * amount of time, in milliseconds.
     *
     * @param lineNumber the source line for this command.
     * @param waitTime the amoutn of time to pause in milliseconds.
     */
    public WaitCommand(int lineNumber, long waitTime)
    {
        super(lineNumber);

        this.waitTime = waitTime;
    }

    /**
     * Executes the Command's function.  The SimulationEngine will call this 
     * method in order to activate the Command and effect change on either the
     * Robot's state or the state of the simulation.
     *
     * @exception thrown if an error occurs attempting to execute the command.
     */
    public void Execute() throws Exception
    {
        Thread currentThread = Thread.currentThread();
        
        try
        {
            currentThread.sleep(waitTime);
        }
        catch (InterruptedException e) {}
    }
}